@@ -3,10 +3,10 @@
  *
  * Type definitions and function declarations relating to date and time.
  *
- * $Id: time.h,v f2287469341d 2017/12/15 20:58:19 keith $
+ * $Id: time.h,v 7a19e2864602 2018/10/18 08:50:58 keith $
  *
  * Written by Colin Peters <colin@bird.fu.is.saga-u.ac.jp>
- * Copyright (C) 1997-2007, 2011, 2015, 2016, MinGW.org Project.
+ * Copyright (C) 1997-2007, 2011, 2015-2018, MinGW.org Project.
  *
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -438,10 +438,24 @@ int nanosleep( const struct timespec *period, struct timespec *residual )
  */
 typedef struct __clockid__ *clockid_t;
 
+/* POSIX prefers to have the standard clockid_t entities defined
+ * as macros, each of which represents an entity of type clockid_t.
+ * Since this is not an integer data type, POSIX does not strictly
+ * require such macros to expand to constant expressions; however,
+ * some ill-behaved applications, (GCC's Ada implementation is one
+ * such), depend on such expansions.  Thus, although it will incur
+ * a small additional run-time overhead to interpret them, we map
+ * such entities in terms of pseudo-pointer references, (which we
+ * discriminate from real pointer references, which we assume to
+ * be always to even valued addresses, by forcing odd values for
+ * the pseudo-pointer references).
+ */
+#define __MINGW_POSIX_CLOCKAPI(ID)  ((clockid_t)(1 + ((ID) << 1)))
+
 /* The standard clockid_t entities which we choose to support.
  */
-extern clockid_t CLOCK_REALTIME;
-extern clockid_t CLOCK_MONOTONIC;
+#define CLOCK_REALTIME  __MINGW_POSIX_CLOCKAPI (0)
+#define CLOCK_MONOTONIC __MINGW_POSIX_CLOCKAPI (1)
 
 /* Prototypes for the standard POSIX functions which provide the
  * API to these standard clockid_t entities.
